#!/usr/bin/php
<?php
// general hint:
//  The number of strokes regarded in this script may be lower than the one on blitzortung/lightningmaps.
//  As it is all about the correlation of signals triggered by the different channels of the onw System Blue,
//  only strokes are regarded which could be aligned to the own signals.
//  This means that strokes which happened at times when boUdpParser.py has not been active, wont count.

// hint:
//  When you code and you are getting some strange messages from the jpgraph modules,
//  it could well be a coding error on your side which is caught by jpgraph.
//  In order to narrow down coding errors, you temporarily may want to comment jpgraph includes.
require_once ('boConfig.php');
require_once ('jpgraph/jpgraph.php');
require_once ('jpgraph/jpgraph_bar.php');
require_once ('jpgraph/jpgraph_line.php');
require_once ('jpgraph/jpgraph_date.php');
require_once ('jpgraph/jpgraph_log.php');
require_once ('jpgraph/jpgraph_scatter.php');
require_once ('boDb.php');

define("CHANNEL_NAME", array("1A", "1B", "1C", "2A")); // symbolic names of the channels of System Blue
define("CHANNEL_COLOR", array('red', 'green', 'yellow', 'magenta')); // colors of the channels of System Blue
define("DARK", ":0.8"); // color modifier, use only one of these
define("LIGHT", ":1.5"); // color modifier, use only one of these
define("OPAQUE", "@0.8"); // color modifier, use only one of these

// This function returns a time in seconds since 1970.01.01 00:00:00 UTC.
// The start-time is aligned to the $step width and set back from now by $range.
// in:
//  $range: time in seconds, default 24 hours
//  $step: time in seconds, default 15 minutes
// out:
//  start-time in seconds since 1970.01.01 00:00:00 UTC
function getStartTime($range = 24 * 60 * 60, $step = 15 * 60) {
    $time = gmmktime(); // UTC now
    $time = floor($time / $step) * $step; // align to the previous step
    $time -= $range;
    return($time );
}

// This function is basically used for testing/debugging purpose.
// It is triggered b calling the script with parameter -d0.
// in:
//  $db: database with type boDb
// out:
//  text on the stdout
function printRawData($db) {
    //$data = $db->getSumOfStrokesPerChannel();
    //$data = $db->getSumOfSignalsPerChannel();
    $data = $db->getStrokePositionForAllChannels();
    print_r($data);
    //$time = getStartTime();
    //print_r("$time\n");
}

// This function prints a four color bargraph with one bar for each channel.
// in:
//  $displayData: the data to be displayed in an array with count(CHANNEL_NUMBER) entries
//  $width: width of the diagram in pixels
//  $height: height of the diagram in pixels
//  $title: title text of the diagram
//  $subtitle: subtitle text of the diagram
//  $xaxis: label text of the xaxis
//  $yaxis: label text of the yaxis
// out:
//  PNG graphics on stdout
// hint:
//  1) Redirect the output to a file.
//  2) When you fid the PNG to be invalid, check whether you output anything not belonging to it.
//     Do not use echo, print, print_r, .. like functions.
//  3) If still not ok, verify none of the included files emits some unwanted character.
//     My first 'failure' was a line break (character 0x0a) at the end of the included 'boDb.php'.
function bgGenericChannelBargraph($displayData, $width, $height, $title, $subtitle, $xaxis, $yaxis) {
    // prepare stacked graph for colored single bars
    $bpdata = array(); // data for a stacked bar plot, an array of arrays, where [0][0] is CHANNEL_NUMBER[0], ... [n][n] is CHANNEL_NUMBER[n], all other [x][y] are 0
    $channelValues = array();
    for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
        $bpdata[$i] = array();
        for($j=0; $j<count(CHANNEL_NUMBER); $j++) {
            if ($i == $j) {
                $bpdata[$i][$j] = $displayData[$i];
                $channelValues[$i] = round($displayData[$i], 1); // text for xaxis
            } else {
                $bpdata[$i][$j] = 0;
            }
        }
    }

    // create the graph. These two calls are always required
    $graph = new Graph($width, $height);
    $graph->ClearTheme();
    $graph->SetScale("textlin");

    // create one bar plot for each channel
    $bp = array(); // one bar plott for each channel
    for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
        $c = CHANNEL_NUMBER[$i];
        $bp[$i] = new BarPlot($bpdata[$i]);
        $bp[$i]->SetFillColor(CHANNEL_COLOR[$c].DARK);
        $bp[$i]->SetLegend(CHANNEL_NAME[$c]);
    }

    // create the accumulated bar plot
    $gbplot = new AccBarPlot($bp);

    // add the accumulated plot to the graph
    $graph->Add($gbplot);

    // decoration
    $graph->title->Set($title);
    $graph->subtitle->Set($subtitle);
    $graph->xaxis->title->Set($xaxis);
    $graph->xaxis->SetTickLabels($channelValues);
    $graph->yaxis->title->Set($yaxis);
    $graph->title->SetFont(FF_FONT1, FS_BOLD);
    $graph->subtitle->SetFont(FF_FONT1);
    $graph->yaxis->title->SetFont(FF_FONT1, FS_BOLD);
    $graph->xaxis->title->SetFont(FF_FONT1, FS_BOLD);

    $graph->legend->Pos(0.0, 0.90, "left", "top");
    $graph->legend->SetLayout(LEGEND_HOR);
    $graph->legend->SetReverse();

    // display the graph
    $graph->Stroke();
}

// This function collects data and triggers a bar graph generation.
// For all channels the ratio between "strokes(channel)/num(strokes)" is calculated.
// in:
//  $db: database with type boDb
//  $width: width of the diagram in pixels
//  $height: height of the diagram in pixels
// out:
//  PNG graphics on stdout
function bgStrokesPerChannelVsTotalStrokes($db, $width, $height) {
    // collect raw data
    $strokes = $db->getSumOfStrokesPerChannel();

    // calculate data to be displayed
    $displayData = array();
    for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
        $c = CHANNEL_NUMBER[$i];
        if (0 != $strokes["total"]) {
            $displayData[$i] = $strokes["sum$c"] / $strokes["total"] * 100; // calculate percentage
        } else {
            $displayData[$i] = 0; // avoid divide by 0
        }
    }

    // create the graph
    bgGenericChannelBargraph($displayData, $width, $height,
        "strokes(channel)/num(strokes)",
        "high is good",
        "channel",
        "percentage [%]"
        );
}

// This function collects data and triggers a bar graph generation.
// For all channels the ratio between "strokes(channel)/signals(channel)" is calculated.
// in:
//  $db: database with type boDb
//  $width: width of the diagram in pixels
//  $height: height of the diagram in pixels
// out:
//  PNG graphics on stdout
function bgStrokesPerChannelVsSignalsPerChannel($db, $width, $height) {
    // collect raw data
    $strokes = $db->getSumOfStrokesPerChannel();
    $signals = $db->getSumOfSignalsPerChannel(); // number of signals per channel

    // calculate data to be displayed
    $displayData = array();
    for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
        $c = CHANNEL_NUMBER[$i];
        if (0 != $signals["sum$c"]) {
            $displayData[$i] = $strokes["sum$c"] / $signals["sum$c"] * 100; // calculate percentage
        } else {
            $displayData[$i] = 0; // avoid divide by 0
        }
    }

    // create the graph
    bgGenericChannelBargraph($displayData, $width, $height,
        "strokes(channel)/signals(channel)",
        "high is good",
        "channel",
        "percentage [%]"
        );
}

// This function collects data and triggers a bar graph generation.
// For all channels the difference between "signals(channel)-strokes(channel)" is calculated.
// in:
//  $db: database with type boDb
//  $width: width of the diagram in pixels
//  $height: height of the diagram in pixels
// out:
//  PNG graphics on stdout
function bgSignalsPerChannelMinusStrokesPerChannel_aka_Noise($db, $width, $height) {
    // collect raw data
    $strokes = $db->getSumOfStrokesPerChannel();
    $signals = $db->getSumOfSignalsPerChannel(); // number of signals per channel

    // calculate data to be displayed
    $displayData = array();
    for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
        $c = CHANNEL_NUMBER[$i];
        $displayData[$i] = ($signals["sum$c"] - $strokes["sum$c"]) / 1000; // calculate noise
    }

    // create the graph
    bgGenericChannelBargraph($displayData, $width, $height,
        "signals(channel)-strokes(channel)",
        "low is good",
        "channel",
        "# bad signals (noise) [1k]"
        );
}

// This function collects data and triggers a bar graph generation.
// For all channels the number of "strokes(channel)" is calculated.
// in:
//  $db: database with type boDb
//  $width: width of the diagram in pixels
//  $height: height of the diagram in pixels
// out:
//  PNG graphics on stdout
function bgStrokesPerChannel_aka_GoodSignals($db, $width, $height) {
    // collect raw data
    $strokes = $db->getSumOfStrokesPerChannel();

    // calculate data to be displayed
    $displayData = array();
    for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
        $c = CHANNEL_NUMBER[$i];
        $displayData[$i] = $strokes["sum$c"] / 1000; // assign good signals
    }

    // create the graph
    bgGenericChannelBargraph($displayData, $width, $height,
        "strokes(channel)",
        "high is good",
        "channel",
        "# good signals [1k]"
        );
}

// This function prints a line graph with one line for the y-axis and one area for the y2-axis for one channel.
// The x-axis has a time base.
// in:
//  $xdata: the time stamps to be displayed for the x-axis
//  $ydata: the data to be displayed as line for the y-axis
//  $ymax: maximum for the y-axis; used in order to make the diagrams for the different channels comparable
//  $y2data: the data to be displayed as line for the y2-axis
//  $y2max: maximum for the y2-axis; used in order to make the diagrams for the different channels comparable
//  $color; color fo the line and area for the y-axis and y2-axis
//  $width: width of the diagram in pixels
//  $height: height of the diagram in pixels
//  $title: title text of the diagram
//  $subtitle: subtitle text of the diagram
//  $xaxis: label text of the xaxis
//  $yaxis: label text of the yaxis
// out:
//  PNG graphics on stdout
function genericTwoAxesLineGraph($xdata, $ydata, $ymax, $y2data, $y2max, $color, $width, $height, $title, $subtitle, $xaxis, $yaxis) {
    // Create the graph and set a scale.
    // These two calls are always required
    $graph = new Graph($width, $height);
    $graph->ClearTheme(); 
    $graph->SetScale('datlin', 0, $ymax);
    $graph->SetY2Scale("lin", 0, $y2max);

    // Setup margin and titles
    $graph->SetMargin(40, 60, 20, 60); // left, right, top, bottom
    $graph->title->Set($title);
    $graph->subtitle->Set($subtitle);
    $graph->xaxis->title->Set($xaxis);
    $graph->yaxis->title->Set($yaxis);
    $graph->title->SetFont(FF_FONT1, FS_BOLD);
    $graph->subtitle->SetFont(FF_FONT1);
    $graph->yaxis->title->SetFont(FF_FONT1, FS_BOLD);
    $graph->xaxis->title->SetFont(FF_FONT1, FS_BOLD);

    // set date time format to hh:mm
    $graph->xaxis->scale->SetDateFormat('H:i');
    // Adjust the start/end to a specific alignment
    $graph->xaxis->scale->SetTimeAlign(MINADJ_15);

    $graph->legend->Pos(0.02, 0.92, "left", "top");
    $graph->legend->SetLayout(LEGEND_HOR);
    $graph->legend->SetReverse();

    // Create the linear plot Y
    $lp = new LinePlot($ydata, $xdata);
    $lp->SetColor($color.DARK);
    $lp->SetLegend("stroke rate");

    // Create the linear plot Y2
    $lp2 = new LinePlot($y2data, $xdata);
    $lp2->SetColor($color.LIGHT);
    $lp2->SetFillColor($color.OPAQUE);
    $lp2->SetLegend("signals");

    // Add the plot to the graph
    $graph->Add($lp);
    $graph->AddY2($lp2);

    // Display the graph
    $graph->Stroke();
}

// This function collects data and triggers a two-axis line graph generation for one single channel.
// The data is collected 24hrs back with steps of 15 minutes.
// On the y-axis, the ratio between "strokes(channel)/signals(channel)" is shown.
// On the y2-axis, the number of "signals(channel)" is shown.
// in:
//  $db: database with type boDb
//  $width: width of the diagram in pixels
//  $height: height of the diagram in pixels
//  $channel: the channel of which the data shall be collected
// out:
//  PNG graphics on stdout
function lgSignalsAndStrokePercentage24h($db, $width, $height, $channel) {
    if (!in_array($channel, CHANNEL_NUMBER)) {
        JpGraphError::Raise("channel $channel is not in CHANNEL_NUMBER, update boConfig.php or remove generation -d5 -c$channel from boGenerateImages.py");
    }
    date_default_timezone_set('UTC');
    $xdata = array();
    $ydata = array();
    $y2data= array();
    $y2max = 0;
    $color = CHANNEL_COLOR[$channel];
    $startTime = getStartTime();
    $steps = 24 * 4; // 24 hours, 4 quarters each
    $quarter = 15 * 60; // 15 minutes, 60 seconds each
    $startDate = new DateTime();
    $startDate->setTimeStamp($startTime);
    $startDate->setTimezone(new DateTimeZone('UTC'));
    $endDate = new DateTime();
    $endDate->setTimeStamp($startTime + ($steps * $quarter));
    $endDate->setTimezone(new DateTimeZone('UTC'));

    for ($s=0; $s<$steps; $s++) {
        $xdata[$s] = $startTime;
        $endTime = $startTime + $quarter;
        $start_ns = "$startTime"."000000000";
        $end_ns = "$endTime"."000000000";
        // collect raw data
        $strokes = $db->getSumOfStrokesPerChannel($start_ns, $end_ns);
        $signals = $db->getSumOfSignalsPerChannel($start_ns, $end_ns);

        // calculate data to be displayed
        if ($signals["sum$channel"]) {
            $ydata[] = $strokes["sum$channel"] / $signals["sum$channel"] * 100; // calculate percentage
            $y2data[] = $signals["sum$channel"];
        } else {
            $ydata[] = 0; // avoid devide by 0
            $y2data[] = 0;
        }
        for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
            $c = CHANNEL_NUMBER[$i];
            if ($y2max < $signals["sum$c"]) {
                $y2max = $signals["sum$c"];
            }
        }

        // next step 15 minutes later
        $startTime = $endTime;
    }

    // create the graph
    genericTwoAxesLineGraph($xdata, $ydata, 100, $y2data, $y2max, $color, $width, $height,
        "signals(".CHANNEL_NAME[$channel]."), strokes(".CHANNEL_NAME[$channel].")/signals(".CHANNEL_NAME[$channel].")",
        $startDate->format("Y-m-d H:i") . "  -  " . $endDate->format("Y-m-d H:i"),
        "time [UTC]",
        "percentage [%]"
    );
}

// This function prints a line graph with one line per channel for the y-axis and one area for the y2-axis.
// The x-axis has an integer base.
// in:
//  $xdata: the data to be displayed for the x-axis
//  $ydata: the data to be displayed as line for the y-axis
//  $width: width of the diagram in pixels
//  $height: height of the diagram in pixels
//  $title: title text of the diagram
//  $subtitle: subtitle text of the diagram
//  $xaxis: label text of the xaxis
//  $yaxis: label text of the yaxis
// out:
//  PNG graphics on stdout
function genericOneAxesLineGraph($xdata, $ydata, $width, $height, $title, $subtitle, $xaxis, $yaxis) {
    // Create the graph and set a scale.
    // These two calls are always required
    $graph = new Graph($width, $height);
    $graph->ClearTheme(); 
    $graph->SetScale('intlog');

    // Setup margin and titles
    $graph->SetMargin(40, 60, 20, 60); // öeft, right, top, bottom
    $graph->title->Set($title);
    $graph->subtitle->Set($subtitle);
    $graph->xaxis->title->Set($xaxis);
    $graph->yaxis->title->Set($yaxis);
    $graph->title->SetFont(FF_FONT1, FS_BOLD);
    $graph->subtitle->SetFont(FF_FONT1);
    $graph->yaxis->title->SetFont(FF_FONT1, FS_BOLD);
    $graph->xaxis->title->SetFont(FF_FONT1, FS_BOLD);

    $graph->legend->Pos(0.02, 0.92, "left", "top");
    $graph->legend->SetLayout(LEGEND_HOR);

    $lp = array();
    for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
        $c = CHANNEL_NUMBER[$i];
        // Create the linear plot Y
        $lp[$i] = new LinePlot($ydata[$i], $xdata);
        $lp[$i]->SetColor(CHANNEL_COLOR[$c].DARK);
        $lp[$i]->SetLegend(CHANNEL_NAME[$c]);
        // Add the plot to the graph
        $graph->Add($lp[$i]);
    }

    // Display the graph
    $graph->Stroke();
}

// This function collects data and triggers a one-axis line graph generation for all channels.
// The data is collected for all times recorded.
// On the xaxis, distance is shown.
// On the yaxis, the number of "strokes(channel)" is shown.
// in:
//  $db: database with type boDb
//  $width: width of the diagram in pixels
//  $height: height of the diagram in pixels
// out:
//  PNG graphics on stdout
function lgStrokesPerDistanceAndChannel($db, $width, $height) {
    $stepWidth = 100; // step width of the distance in km
    $maxDistance = $db->getMaxDistanceStroke();
    $maxDistance = $maxDistance["max(distance)"];
    $xdata = array();
    $ydata = array();
    for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
        $ydata[$i] = array();
    }
    for($d=0; $d<$maxDistance; $d+=$stepWidth) {
        $xdata[] = $d;

        // collect raw data
        $strokes = $db->getSumOfStrokesPerDistancePerChannel($d, $d + $stepWidth);
        for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
            $c = CHANNEL_NUMBER[$i];
            if ($strokes["sum$c"]) {
                $ydata[$i][] = $strokes["sum$c"];
            } else {
                $ydata[$i][] = 0;
            }
        }
    }

    // create the graph
    genericOneAxesLineGraph($xdata, $ydata, $width, $height,
        "strokes per channel and distance",
        "max distance = $maxDistance",
        "distance [km]",
        "count [1]"
    );
}

// This function collects data and triggers a scatter graph generation for all channels.
// The data is collected for all times recorded.
// On the xaxis/yaxis are the pixles fro bottom left of a map as background.
// The properties of the map have to be defined as 
//  "MAP", "MAP_WIDTH", "MAP_HEIGHT", "MAP_LON_LEFT", "MAP_LON_RIGHT", "MAP_LAT_BOTTOM".
//  See the corresponding definitions at the top of the file.
// in:
//  $db: database with type boDb
// out:
//  PNG graphics on stdout
function mapChannelStrokes($db) {
    // constants to fit lat/lon to the background image
    $mapWidth = MAP_WIDTH;
    $mapHeight = MAP_HEIGHT;

    $mapLonLeft = MAP_LON_LEFT;
    $mapLonRight = MAP_LON_RIGHT;
    $mapLonDelta = $mapLonRight - $mapLonLeft;

    $mapLatBottom = MAP_LAT_BOTTOM;
    $mapLatBottomDegree = $mapLatBottom * M_PI / 180;

    $worldMapWidth = (($mapWidth / $mapLonDelta) * 360) / (2 * M_PI);
    $mapOffsetY = ($worldMapWidth / 2 * log((1 + sin($mapLatBottomDegree)) / (1 - sin($mapLatBottomDegree))));

    // Data arrays
    $data = $db->getStrokePositionForAllChannels();
    $channelData = array();
    $xdata = array();
    $ydata = array();
    for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
        $channelData[$i] = array();
        $xdata[$i] = array();
        $ydata[$i] = array();
    }

    for($i=0; $i<count($data); $i++) {
        $lat = $data[$i]["lat"];
        $lon = $data[$i]["lon"];
        $x = intval(($lon - $mapLonLeft) * ($mapWidth / $mapLonDelta));
        $y = intval((($worldMapWidth / 2 * log((1 + sin($lat * M_PI / 180)) / (1 - sin($lat * M_PI / 180)))) - $mapOffsetY));
        for($j=0; $j<count(CHANNEL_NUMBER); $j++) {
            $c = CHANNEL_NUMBER[$j];
            if (1 == $data[$i]["channel$c"]) {
                $channelData[$j][] = array("x" => $x, "y" => $y);
            }
        }
    }

    // eliminate duplicates
    for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
        $channelData[$i] = array_unique($channelData[$i], SORT_REGULAR);
        foreach ($channelData[$i] as $coordinate) {
            $xdata[$i][] = $coordinate["x"];
            $ydata[$i][] = $coordinate["y"];
        }
    }

    // Setup the graph
    $graph = new Graph(MAP_WIDTH, MAP_HEIGHT);
    $graph->ClearTheme(); 

    // We add no margin so the plot area
    $graph->img->SetMargin(0, 0, 0, 0);
    $graph->SetScale('linlin', 0, MAP_HEIGHT-1, 0, MAP_WIDTH-1); // $aAxisType, $aYMin, $aYMax, $aXMin, $aXMax

    // We don't want any axis to be shown
    $graph->xaxis->Hide();
    $graph->yaxis->Hide();

    // Use a worldmap as the background and let it fill the plot area
    $graph->SetBackgroundImage(MAP, BGIMG_FILLPLOT);

    $graph->legend->Pos(0.02, 0.97, "left", "bottom");
    $graph->legend->SetLayout(LEGEND_HOR);

    $sp = array();
    $spCount = 0;
    for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
        $c = CHANNEL_NUMBER[$i];
        // eliminate empty channel data
        if (count($ydata[$i]) and count($xdata[$i]))
        {
            // Finally create the scatterplot
            $sp[$spCount] = new ScatterPlot($ydata[$i], $xdata[$i]);
            $sp[$spCount]->SetLegend(CHANNEL_NAME[$c]);

            // Configure the markers
            $sp[$spCount]->mark->SetSize(0); // 0 is still one pixel
            $sp[$spCount]->mark->SetColor(CHANNEL_COLOR[$c]);
            $sp[$spCount]->mark->SetFillColor(CHANNEL_COLOR[$c]);
            $spCount += 1;
        }
    }

    // Add it to the graph ...
    if (count($sp)) {
        // ... but only if there is data available
        $graph->Add($sp);
    }

    // .. and output to browser
    $graph->Stroke();
}

// This function pareses the command line parameters and assigns default values for non-present parameters.
// in: -
// out:
//   1) In case of success: An array with the command line parameters as key and the assigned values.
//   2) Else: The intended usage is shown on stdout. The program terminates.
function parseOptions() {
    // define the options
    $shortopts  = "d:"; // diagram number
    $shortopts .= "w:"; // width in pixel
    $shortopts .= "h:"; // height in pixel
    $shortopts .= "c:"; // channel number
    $longopts  = array("help");

    // get the options from the command line
    $options = getopt($shortopts, $longopts);

    // convert to integer if present, else set default values
    if (array_key_exists('d', $options)) {
        $options['d'] = intval($options['d']);
    } else {
        $options['d'] = 0;
    }
    if (array_key_exists('w', $options)) {
        $options['w'] = intval($options['w']);
    } else {
        $options['w'] = 640;
    }
    if (array_key_exists('h', $options)) {
        $options['h'] = intval($options['h']);
    } else {
        $options['h'] = 480;
    }
    if (array_key_exists('c', $options)) {
        $options['c'] = intval($options['c']);
    } else {
        $options['c'] = 0;
    }

    // print help screen
    if (4 != count($options)) {
        $help  = "-d<diagram number> default: 0\n";
        $help .= "  0 = text output of the raw data\n";
        $help .= "  1 = strokes(channel)/num(strokes)\n";
        $help .= "  2 = strokes(channel)/signals(channel)\n";
        $help .= "  3 = signals(channel)-strokes(channel)\n";
        $help .= "  4 = strokes(channel)\n";
        $help .= "  5 -c<[0..3]> = signals(c), strokes(c)/signals(c), 24h\n";
        $help .= "  6 = strokes per distance per channel\n";
        $help .= "  7 = map channel strokes\n";
        $help .= "-w<width in pixels> default: 640\n";
        $help .= "-h<height in pixels> default: 320\n";
        die($help);
    }
    return($options);
}

// This function acts as a dispatcher for the commands received from the command line.
function main() {
    // get the command line options and assign them to nicely named variables
    $options = parseOptions();
    $diagram = $options["d"];
    $width = $options["w"];
    $height = $options["h"];
    $channel = $options["c"];

    // open an instance of the database (which will automatically close when leavng main)
    $db = new boDb();

    // command dispatcher
    switch ($diagram) {
        case 0: printRawData($db); break;
        case 1: bgStrokesPerChannelVsTotalStrokes($db, $width, $height); break;
        case 2: bgStrokesPerChannelVsSignalsPerChannel($db, $width, $height); break;
        case 3: bgSignalsPerChannelMinusStrokesPerChannel_aka_Noise($db, $width, $height); break;
        case 4: bgStrokesPerChannel_aka_GoodSignals($db, $width, $height); break;
        case 5: lgSignalsAndStrokePercentage24h($db, $width, $height, $channel); break;
        case 6: lgStrokesPerDistanceAndChannel($db, $width, $height); break;
        case 7: mapChannelStrokes($db); break; // fixed size determined by the background image
        default:
            print("diagram " . $diagram . " is not implemented\n");
    }
}

// the main function is called
main();
?>
